@@ -7,6 +7,10 @@ Exceptions used in gspread.
 """
 
 
+class UnSupportedExportFormat(Exception):
+    """Raised when export format is not supported."""
+
+
 class GSpreadException(Exception):
     """A base class for gspread's exceptions."""
 
@@ -31,6 +35,10 @@ class IncorrectCellLabel(GSpreadException):
     """The cell label is incorrect."""
 
 
+class InvalidInputValue(GSpreadException):
+    """The provided values is incorrect."""
+
+
 class APIError(GSpreadException):
     def __init__(self, response):
 
