@@ -27,15 +27,18 @@ A1_ADDR_ROW_COL_RE = re.compile(r"([A-Za-z]+)?([1-9]\d*)?$")
 URL_KEY_V1_RE = re.compile(r"key=([^&#]+)")
 URL_KEY_V2_RE = re.compile(r"/spreadsheets/d/([a-zA-Z0-9-_]+)")
 
-Dimension = namedtuple("_Dimension", ["rows", "cols"])("ROWS", "COLUMNS")
+Dimension = namedtuple("Dimension", ["rows", "cols"])("ROWS", "COLUMNS")
 ValueRenderOption = namedtuple(
-    "_ValueRenderOption", ["formatted", "unformatted", "formula"]
+    "ValueRenderOption", ["formatted", "unformatted", "formula"]
 )("FORMATTED_VALUE", "UNFORMATTED_VALUE", "FORMULA")
-ValueInputOption = namedtuple("_ValueInputOption", ["raw", "user_entered"])(
+ValueInputOption = namedtuple("ValueInputOption", ["raw", "user_entered"])(
     "RAW", "USER_ENTERED"
 )
+DateTimeOption = namedtuple("DateTimeOption", ["serial_number", "formated_string"])(
+    "SERIAL_NUMBER", "FORMATTED_STRING"
+)
 MimeType = namedtuple(
-    "_MimeType",
+    "MimeType",
     ["google_sheets", "pdf", "excel", "csv", "open_office_sheet", "tsv", "zip"],
 )(
     "application/vnd.google-apps.spreadsheet",
@@ -47,7 +50,7 @@ MimeType = namedtuple(
     "application/zip",
 )
 ExportFormat = namedtuple(
-    "_ExportFormat", ["PDF", "EXCEL", "CSV", "OPEN_OFFICE_SHEET", "TSV", "ZIPPED_HTML"]
+    "ExportFormat", ["PDF", "EXCEL", "CSV", "OPEN_OFFICE_SHEET", "TSV", "ZIPPED_HTML"]
 )(
     MimeType.pdf,
     MimeType.excel,
@@ -57,6 +60,31 @@ ExportFormat = namedtuple(
     MimeType.zip,
 )
 
+PasteType = namedtuple(
+    "PasteType",
+    [
+        "normal",
+        "values",
+        "format",
+        "no_borders",
+        "formula",
+        "data_validation",
+        "conditional_formating",
+    ],
+)(
+    "PASTE_NORMAL",
+    "PASTE_VALUES",
+    "PASTE_FORMAT",
+    "PASTE_NO_BORDERS",
+    "PASTE_FORMULA",
+    "PASTE_DATA_VALIDATION",
+    "PASTE_CONDITIONAL_FORMATTING",
+)
+
+PasteOrientation = namedtuple("PasteOrientation", ["normal", "transpose"])(
+    "NORMAL", "TRANSPOSE"
+)
+
 
 def convert_credentials(credentials):
     module = credentials.__module__
