@@ -11,6 +11,8 @@ from .exceptions import GSpreadException
 from .urls import SPREADSHEET_URL, WORKSHEET_DRIVE_URL
 from .utils import (
     Dimension,
+    PasteOrientation,
+    PasteType,
     ValueInputOption,
     ValueRenderOption,
     a1_range_to_grid_range,
@@ -139,6 +141,12 @@ class Worksheet:
         """Worksheet index."""
         return self._properties["index"]
 
+    @property
+    def isSheetHidden(self):
+        """Worksheet hidden status."""
+        # if the property is not set then hidden=False
+        return self._properties.get("hidden", False)
+
     @property
     def updated(self):
         """.. deprecated:: 2.0
@@ -205,9 +213,7 @@ class Worksheet:
         :param value_render_option: (optional) Determines how values should be
                                     rendered in the output. See
                                     `ValueRenderOption`_ in the Sheets API.
-        :type value_render_option:  ( `ValueRenderOption.formatted` |
-                                    `ValueRenderOption.unformatted` |
-                                    `ValueRenderOption.formula` )
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
 
         .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
 
@@ -231,9 +237,7 @@ class Worksheet:
         :param value_render_option: (optional) Determines how values should be
                                     rendered in the output. See
                                     `ValueRenderOption`_ in the Sheets API.
-        :type value_render_option:  ( `ValueRenderOption.formatted` |
-                                    `ValueRenderOption.unformatted` |
-                                    `ValueRenderOption.formula` )
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
 
         .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
 
@@ -346,6 +350,7 @@ class Worksheet:
         :param str major_dimension: (optional) The major dimension of the
             values. `Dimension.rows` ("ROWS") or `Dimension.cols` ("COLUMNS").
             Defaults to Dimension.rows
+        :type major_dimension: :namedtuple:`~gspread.utils.Dimension`
 
         :param str value_render_option: (optional) Determines how values should
             be rendered in the output. See `ValueRenderOption`_ in
@@ -368,12 +373,31 @@ class Worksheet:
                 the formulas. For example, if A1 is 1.23 and A2 is =A1 and
                 formatted as currency, then A2 would return "=A1".
 
-        .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+            .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
+
 
         :param str date_time_render_option: (optional) How dates, times, and
-            durations should be represented in the output. This is ignored if
-            ``value_render_option`` is ``ValueRenderOption.formatted``.
-            The default ``date_time_render_option`` is ``SERIAL_NUMBER``.
+            durations should be represented in the output.
+
+            Possible values are:
+
+            ``DateTimeOption.serial_number``
+                (default) Instructs date, time, datetime, and duration fields
+                to be output as doubles in "serial number" format,
+                as popularized by Lotus 1-2-3.
+
+            ``DateTimeOption.formated_string``
+                Instructs date, time, datetime, and duration fields to be output
+                as strings in their given number format
+                (which depends on the spreadsheet locale).
+
+            .. note::
+
+                This is ignored if ``value_render_option`` is ``ValueRenderOption.formatted``.
+
+            The default ``date_time_render_option`` is ``DateTimeOption.serial_number``.
+        :type date_time_render_option: :namedtuple:`~gspread.utils.DateTimeOption`
 
         .. note::
 
@@ -458,9 +482,10 @@ class Worksheet:
         :param list numericise_ignore: (optional) List of ints of indices of
             the columns (starting at 1) to ignore numericising, special use
             of ['all'] to ignore numericising on all columns.
-        :param str value_render_option: (optional) Determines how values should
+        :param value_render_option: (optional) Determines how values should
             be rendered in the output. See `ValueRenderOption`_ in
             the Sheets API.
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
 
         :param list expected_headers: (optional) List of expected headers, they must be unique.
 
@@ -533,11 +558,52 @@ class Worksheet:
         Empty cells in this list will be rendered as :const:`None`.
 
         :param int row: Row number (one-based).
-        :param str value_render_option: (optional) Determines how values should
+        :param value_render_option: (optional) Determines how values should
             be rendered in the output. See `ValueRenderOption`_ in
             the Sheets API.
 
-        .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+            Possible values are:
+
+            ``ValueRenderOption.formatted``
+                (default) Values will be calculated and formatted according
+                to the cell's formatting. Formatting is based on the
+                spreadsheet's locale, not the requesting user's locale.
+
+            ``ValueRenderOption.unformatted``
+                Values will be calculated, but not formatted in the reply.
+                For example, if A1 is 1.23 and A2 is =A1 and formatted as
+                currency, then A2 would return the number 1.23.
+
+            ``ValueRenderOption.formula``
+                Values will not be calculated. The reply will include
+                the formulas. For example, if A1 is 1.23 and A2 is =A1 and
+                formatted as currency, then A2 would return "=A1".
+
+            .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
+
+        :param date_time_render_option: (optional) How dates, times, and
+            durations should be represented in the output.
+
+            Possible values are:
+
+            ``DateTimeOption.serial_number``
+                (default) Instructs date, time, datetime, and duration fields
+                to be output as doubles in "serial number" format,
+                as popularized by Lotus 1-2-3.
+
+            ``DateTimeOption.formated_string``
+                Instructs date, time, datetime, and duration fields to be output
+                as strings in their given number format
+                (which depends on the spreadsheet locale).
+
+            .. note::
+
+                This is ignored if ``value_render_option`` is ``ValueRenderOption.formatted``.
+
+            The default ``date_time_render_option`` is ``DateTimeOption.serial_number``.
+        :type date_time_render_option: :namedtuple:`~gspread.utils.DateTimeOption`
         """
         try:
             data = self.get("A{}:{}".format(row, row), **kwargs)
@@ -554,6 +620,7 @@ class Worksheet:
         :param str value_render_option: (optional) Determines how values should
             be rendered in the output. See `ValueRenderOption`_ in
             the Sheets API.
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
 
         .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
         """
@@ -613,7 +680,7 @@ class Worksheet:
         """Updates many cells at once.
 
         :param list cell_list: List of :class:`gspread.cell.Cell` objects to update.
-        :param str value_input_option: (optional) How the input data should be
+        :param  value_input_option: (optional) How the input data should be
             interpreted. Possible values are:
 
             ``ValueInputOption.raw``
@@ -629,6 +696,8 @@ class Worksheet:
 
             See `ValueInputOption`_ in the Sheets API.
 
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
+
         .. _ValueInputOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueInputOption
 
         Example::
@@ -667,38 +736,76 @@ class Worksheet:
     def get(self, range_name=None, **kwargs):
         """Reads values of a single range or a cell of a sheet.
 
-        :param str range_name: (optional) Cell range in the A1 notation or
-            a named range.
+         :param str range_name: (optional) Cell range in the A1 notation or
+             a named range.
 
-        :param str major_dimension: (optional) The major dimension that results
-            should use. Either ``ROWS`` or ``COLUMNS``.
+         :param str major_dimension: (optional) The major dimension that results
+             should use. Either ``ROWS`` or ``COLUMNS``.
 
-        :param str value_render_option: (optional) How values should be
-            represented in the output. The default render option is
-            ``ValueRenderOption.formatted``.
+         :param value_render_option: (optional) Determines how values should
+             be rendered in the output. See `ValueRenderOption`_ in
+             the Sheets API.
+
+             Possible values are:
+
+             ``ValueRenderOption.formatted``
+                 (default) Values will be calculated and formatted according
+                 to the cell's formatting. Formatting is based on the
+                 spreadsheet's locale, not the requesting user's locale.
+
+             ``ValueRenderOption.unformatted``
+                 Values will be calculated, but not formatted in the reply.
+                 For example, if A1 is 1.23 and A2 is =A1 and formatted as
+                 currency, then A2 would return the number 1.23.
+
+             ``ValueRenderOption.formula``
+                 Values will not be calculated. The reply will include
+                 the formulas. For example, if A1 is 1.23 and A2 is =A1 and
+                 formatted as currency, then A2 would return "=A1".
+
+             .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
 
         :param str date_time_render_option: (optional) How dates, times, and
-            durations should be represented in the output. This is ignored if
-            ``value_render_option`` is ``ValueRenderOption.formatted``. The default
-            ``date_time_render_option`` is ``SERIAL_NUMBER``.
+             durations should be represented in the output.
 
-        :rtype: :class:`gspread.worksheet.ValueRange`
+             Possible values are:
 
-        Examples::
+             ``DateTimeOption.serial_number``
+                 (default) Instructs date, time, datetime, and duration fields
+                 to be output as doubles in "serial number" format,
+                 as popularized by Lotus 1-2-3.
 
-            # Return all values from the sheet
-            worksheet.get()
+             ``DateTimeOption.formated_string``
+                 Instructs date, time, datetime, and duration fields to be output
+                 as strings in their given number format
+                 (which depends on the spreadsheet locale).
 
-            # Return value of 'A1' cell
-            worksheet.get('A1')
+             .. note::
 
-            # Return values of 'A1:B2' range
-            worksheet.get('A1:B2')
+                 This is ignored if ``value_render_option`` is ``ValueRenderOption.formatted``.
 
-            # Return values of 'my_range' named range
-            worksheet.get('my_range')
+             The default ``date_time_render_option`` is ``DateTimeOption.serial_number``.
+        :type date_time_render_option: :namedtuple:`~gspread.utils.DateTimeOption`
 
-        .. versionadded:: 3.3
+         :rtype: :class:`gspread.worksheet.ValueRange`
+
+         Examples::
+
+             # Return all values from the sheet
+             worksheet.get()
+
+             # Return value of 'A1' cell
+             worksheet.get('A1')
+
+             # Return values of 'A1:B2' range
+             worksheet.get('A1:B2')
+
+             # Return values of 'my_range' named range
+             worksheet.get('my_range')
+
+         .. versionadded:: 3.3
         """
         range_name = absolute_range_name(self.title, range_name)
 
@@ -728,14 +835,52 @@ class Worksheet:
         :param str major_dimension: (optional) The major dimension that results
             should use. Either ``ROWS`` or ``COLUMNS``.
 
-        :param str value_render_option: (optional) How values should be
-            represented in the output. The default render option
-            is ``ValueRenderOption.formatted``.
+        :param value_render_option: (optional) Determines how values should
+            be rendered in the output. See `ValueRenderOption`_ in
+            the Sheets API.
+
+            Possible values are:
+
+            ``ValueRenderOption.formatted``
+                (default) Values will be calculated and formatted according
+                to the cell's formatting. Formatting is based on the
+                spreadsheet's locale, not the requesting user's locale.
+
+            ``ValueRenderOption.unformatted``
+                Values will be calculated, but not formatted in the reply.
+                For example, if A1 is 1.23 and A2 is =A1 and formatted as
+                currency, then A2 would return the number 1.23.
+
+            ``ValueRenderOption.formula``
+                Values will not be calculated. The reply will include
+                the formulas. For example, if A1 is 1.23 and A2 is =A1 and
+                formatted as currency, then A2 would return "=A1".
+
+            .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+
+        :type value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
 
         :param str date_time_render_option: (optional) How dates, times, and
-            durations should be represented in the output. This is ignored if
-            value_render_option is ``ValueRenderOption.formatted``. The default dateTime
-            render option is ``SERIAL_NUMBER``.
+            durations should be represented in the output.
+
+            Possible values are:
+
+            ``DateTimeOption.serial_number``
+                (default) Instructs date, time, datetime, and duration fields
+                to be output as doubles in "serial number" format,
+                as popularized by Lotus 1-2-3.
+
+            ``DateTimeOption.formated_string``
+                Instructs date, time, datetime, and duration fields to be output
+                as strings in their given number format
+                (which depends on the spreadsheet locale).
+
+            .. note::
+
+                This is ignored if ``value_render_option`` is ``ValueRenderOption.formatted``.
+
+            The default ``date_time_render_option`` is ``DateTimeOption.serial_number``.
+        :type date_time_render_option: :namedtuple:`~gspread.utils.DateTimeOption`
 
         .. versionadded:: 3.3
 
@@ -795,6 +940,55 @@ class Worksheet:
                 applied when entering text into a cell via
                 the Google Sheets UI.
 
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
+
+        :param response_value_render_option: (optional) Determines how values should
+            be rendered in the output. See `ValueRenderOption`_ in
+            the Sheets API.
+
+            Possible values are:
+
+            ``ValueRenderOption.formatted``
+                (default) Values will be calculated and formatted according
+                to the cell's formatting. Formatting is based on the
+                spreadsheet's locale, not the requesting user's locale.
+
+            ``ValueRenderOption.unformatted``
+                Values will be calculated, but not formatted in the reply.
+                For example, if A1 is 1.23 and A2 is =A1 and formatted as
+                currency, then A2 would return the number 1.23.
+
+            ``ValueRenderOption.formula``
+                Values will not be calculated. The reply will include
+                the formulas. For example, if A1 is 1.23 and A2 is =A1 and
+                formatted as currency, then A2 would return "=A1".
+
+            .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+
+        :type response_value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
+
+        :param str response_date_time_render_option: (optional) How dates, times, and
+            durations should be represented in the output.
+
+            Possible values are:
+
+            ``DateTimeOption.serial_number``
+                (default) Instructs date, time, datetime, and duration fields
+                to be output as doubles in "serial number" format,
+                as popularized by Lotus 1-2-3.
+
+            ``DateTimeOption.formated_string``
+                Instructs date, time, datetime, and duration fields to be output
+                as strings in their given number format
+                (which depends on the spreadsheet locale).
+
+            .. note::
+
+                This is ignored if ``value_render_option`` is ``ValueRenderOption.formatted``.
+
+            The default ``date_time_render_option`` is ``DateTimeOption.serial_number``.
+        :type date_time_render_option: :namedtuple:`~gspread.utils.DateTimeOption`
+
         Examples::
 
             # Sets 'Hello world' in 'A2' cell
@@ -885,6 +1079,55 @@ class Worksheet:
               applied when entering text into a cell via
               the Google Sheets UI.
 
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
+
+        :param response_value_render_option: (optional) Determines how values should
+            be rendered in the output. See `ValueRenderOption`_ in
+            the Sheets API.
+
+            Possible values are:
+
+            ``ValueRenderOption.formatted``
+                (default) Values will be calculated and formatted according
+                to the cell's formatting. Formatting is based on the
+                spreadsheet's locale, not the requesting user's locale.
+
+            ``ValueRenderOption.unformatted``
+                Values will be calculated, but not formatted in the reply.
+                For example, if A1 is 1.23 and A2 is =A1 and formatted as
+                currency, then A2 would return the number 1.23.
+
+            ``ValueRenderOption.formula``
+                Values will not be calculated. The reply will include
+                the formulas. For example, if A1 is 1.23 and A2 is =A1 and
+                formatted as currency, then A2 would return "=A1".
+
+            .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
+
+        :type response_value_render_option: :namedtuple:`~gspread.utils.ValueRenderOption`
+
+        :param str response_date_time_render_option: (optional) How dates, times, and
+            durations should be represented in the output.
+
+            Possible values are:
+
+            ``DateTimeOption.serial_number``
+                (default) Instructs date, time, datetime, and duration fields
+                to be output as doubles in "serial number" format,
+                as popularized by Lotus 1-2-3.
+
+            ``DateTimeOption.formated_string``
+                Instructs date, time, datetime, and duration fields to be output
+                as strings in their given number format
+                (which depends on the spreadsheet locale).
+
+            .. note::
+
+                This is ignored if ``value_render_option`` is ``ValueRenderOption.formatted``.
+
+            The default ``date_time_render_option`` is ``DateTimeOption.serial_number``.
+        :type date_time_render_option: :namedtuple:`~gspread.utils.DateTimeOption`
+
         Examples::
 
             worksheet.batch_update([{
@@ -1313,9 +1556,10 @@ class Worksheet:
         Widens the worksheet if there are more values than columns.
 
         :param list values: List of values for the new row.
-        :param str value_input_option: (optional) Determines how the input data
+        :param value_input_option: (optional) Determines how the input data
             should be interpreted. See `ValueInputOption`_ in the Sheets API
             reference.
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
         :param str insert_data_option: (optional) Determines how the input data
             should be inserted. See `InsertDataOption`_ in the Sheets API
             reference.
@@ -1356,6 +1600,7 @@ class Worksheet:
             should be interpreted. Possible values are ``ValueInputOption.raw``
             or ``ValueInputOption.user_entered``.
             See `ValueInputOption`_ in the Sheets API.
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
         :param str insert_data_option: (optional) Determines how the input data
             should be inserted. See `InsertDataOption`_ in the Sheets API
             reference.
@@ -1399,6 +1644,7 @@ class Worksheet:
             should be interpreted. Possible values are ``ValueInputOption.raw``
             or ``ValueInputOption.user_entered``.
             See `ValueInputOption`_ in the Sheets API.
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
         :param bool inherit_from_before: (optional) If True, the new row will
             inherit its properties from the previous row. Defaults to False,
             meaning that the new row acquires the properties of the row
@@ -1437,6 +1683,7 @@ class Worksheet:
             should be interpreted. Possible values are ``ValueInputOption.raw``
             or ``ValueInputOption.user_entered``.
             See `ValueInputOption`_ in the Sheets API.
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
         :param bool inherit_from_before: (optional) If true, new rows will
             inherit their properties from the previous row. Defaults to False,
             meaning that new rows acquire the properties of the row immediately
@@ -1505,6 +1752,7 @@ class Worksheet:
             should be interpreted. Possible values are ``ValueInputOption.raw``
             or ``ValueInputOption.user_entered``.
             See `ValueInputOption`_ in the Sheets API.
+        :type value_input_option: :namedtuple:`~gspread.utils.ValueInputOption`
         :param bool inherit_from_before: (optional) If True, new columns will
             inherit their properties from the previous column. Defaults to
             False, meaning that new columns acquire the properties of the
@@ -1657,7 +1905,8 @@ class Worksheet:
     def delete_dimension(self, dimension, start_index, end_index=None):
         """Deletes multi rows from the worksheet at the specified index.
 
-        :param str dimension: A dimension to delete. ``Dimension.rows`` or ``Dimension.cols``.
+        :param dimension: A dimension to delete. ``Dimension.rows`` or ``Dimension.cols``.
+        :type dimension: :namedtuple:`~gspread.utils.Dimension`
         :param int start_index: Index of a first row for deletion.
         :param int end_index: Index of a last row for deletion. When
             ``end_index`` is not specified this method only deletes a single
@@ -2299,6 +2548,7 @@ class Worksheet:
         :param int end: The (exclusive) end of the dimension to hide
         :param str dimension: The dimension to hide, can be one of
             ``ROWS`` or ``COLUMNS``.
+        :type diension: :namedtuple:`~gspread.utils.Dimension`
         """
         body = {
             "requests": [
@@ -2353,6 +2603,7 @@ class Worksheet:
         :param int end: The (inclusive) end of the dimension to unhide
         :param str dimension: The dimension to hide, can be one of
             ``ROWS`` or ``COLUMNS``.
+        :type dimension: :namedtuple:`~gspread.utils.Dimension`
         """
         body = {
             "requests": [
@@ -2423,3 +2674,94 @@ class Worksheet:
     def show(self):
         """Show the current worksheet in the UI."""
         return self._set_hidden_flag(False)
+
+    def copy_range(
+        self,
+        source,
+        dest,
+        paste_type=PasteType.normal,
+        paste_orientation=PasteOrientation.normal,
+    ):
+        """Copies a range of data from source to dest
+
+        .. note::
+
+           ``paste_type`` values are explained here: `Paste Types`_
+
+           .. _Paste Types: https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/request#pastetype
+
+        :param str source: The A1 notation of the source range to copy
+        :param str dest: The A1 notation of the destination where to paste the data
+            Can be the A1 notation of the top left corner where the range must be paste
+            ex: G16, or a complete range notation ex: G16:I20.
+            The dimensions of the destination range is not checked and has no effect,
+            if the destination range does not match the source range dimension, the entire
+            source range is copies anyway.
+        :param paste_type: the paste type to apply. Many paste type are available from
+            the Sheet API, see above note for detailed values for all values and their effects.
+            Defaults to ``PasteType.normal``
+        :type paste_type: :namedtuple:`~gspread.utils.PasteType`
+        :param paste_orientation: The paste orient to apply.
+            Possible values are: ``normal`` to keep the same orientation, ``transpose`` where all rows become columns and vice versa.
+        :type paste_orientation: :namedtuple:`~gspread.utils.PasteOrientation`
+        """
+        body = {
+            "requests": [
+                {
+                    "copyPaste": {
+                        "source": a1_range_to_grid_range(source, self.id),
+                        "destination": a1_range_to_grid_range(dest, self.id),
+                        "pasteType": paste_type,
+                        "pasteOrientation": paste_orientation,
+                    }
+                }
+            ]
+        }
+
+        return self.spreadsheet.batch_update(body)
+
+    def cut_range(
+        self,
+        source,
+        dest,
+        paste_type=PasteType.normal,
+    ):
+        """Moves a range of data form source to dest
+
+        .. note::
+
+           ``paste_type`` values are explained here: `Paste Types`_
+
+           .. _Paste Types: https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/request#pastetype
+
+        :param str source: The A1 notation of the source range to move
+        :param str dest: The A1 notation of the destination where to paste the data
+            **it must be a single cell** in the A1 notation. ex: G16
+        :param paste_type: the paste type to apply. Many paste type are available from
+            the Sheet API, see above note for detailed values for all values and their effects.
+            Defaults to ``PasteType.normal``
+        :type paste_type: :namedtuple:`~gspread.utils.PasteType`
+        """
+
+        # in the cut/paste request, the destination object
+        # is a `gridCoordinate` and not a `gridRang`
+        # it has different object keys
+        grid_dest = a1_range_to_grid_range(dest, self.id)
+
+        body = {
+            "requests": [
+                {
+                    "cutPaste": {
+                        "source": a1_range_to_grid_range(source, self.id),
+                        "destination": {
+                            "sheetId": grid_dest["sheetId"],
+                            "rowIndex": grid_dest["startRowIndex"],
+                            "columnIndex": grid_dest["startColumnIndex"],
+                        },
+                        "pasteType": paste_type,
+                    }
+                }
+            ]
+        }
+
+        return self.spreadsheet.batch_update(body)
